package com.persagy.fm.department.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.base.CaseFormat;
import com.google.common.collect.Lists;
import com.persagy.common.enums.ResponseCode;
import com.persagy.common.exception.BusinessException;
import com.persagy.fm.common.constant.DefaultValueConstants;
import com.persagy.fm.common.constant.PageQueryConstants;
import com.persagy.fm.common.constant.enums.BoolEnum;
import com.persagy.fm.common.constant.enums.ValidEnum;
import com.persagy.fm.common.context.DefaultAppContext;
import com.persagy.fm.common.model.dto.Sort;
import com.persagy.fm.department.constant.DepConstants;
import com.persagy.fm.department.constant.enums.DepTypeEnum;
import com.persagy.fm.department.constant.enums.ResourceFromEnum;
import com.persagy.fm.department.dao.DepartmentMapper;
import com.persagy.fm.department.model.ConvertDepartmentTool;
import com.persagy.fm.department.model.Department;
import com.persagy.fm.department.model.dto.AddDepartmentDTO;
import com.persagy.fm.department.model.dto.PageQueryDepartmentDTO;
import com.persagy.fm.department.model.dto.QueryDepartmentDTO;
import com.persagy.fm.department.model.dto.UpdateDepartmentDTO;
import com.persagy.fm.department.model.vo.DepSimpleObjVO;
import com.persagy.fm.department.model.vo.FullPathInfoVO;
import com.persagy.fm.department.service.IDepartmentService;
import com.persagy.fm.depproject.model.dto.AddDepProjectDTO;
import com.persagy.fm.depproject.service.IDepProjectService;
import com.persagy.fm.person.model.dto.PageQueryPersonDTO;
import com.persagy.fm.person.model.vo.ResponsePersonListItemVO;
import com.persagy.fm.person.service.IPersonQueryService;
import com.persagy.fm.workresume.model.WorkResume;
import com.persagy.fm.workresume.model.dto.QueryWorkResumeDTO;
import com.persagy.fm.workresume.service.IWorkResumeService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 部门(Department) service层
 *
 * @author lixing
 * @version V1.0 2021-03-11 15:10:35
 */
@Service
public class DepartmentServiceImpl extends ServiceImpl<DepartmentMapper, Department>
        implements IDepartmentService {
    @Autowired
    private IDepProjectService depProjectService;
    @Autowired
    private IPersonQueryService personQueryService;
    @Autowired
    private IWorkResumeService workResumeService;

    @Override
    @Transactional
    public String createDepartment(AddDepartmentDTO addDepartmentDTO) {
        Department department = saveDepartment(addDepartmentDTO);
        // 保存部门和项目的关系
        List<String> projectIds = addDepartmentDTO.getProjectIds();
        String depId = department.getId();
        saveDepProject(projectIds, depId);

        return depId;
    }

    @Override
    public String createDepartment(Department department) {
        save(department);
        return department.getId();
    }

    private void saveDepProject(List<String> projectIds, String depId) {
        if (CollectionUtils.isEmpty(projectIds)) {
            return;
        }

        List<AddDepProjectDTO> depProjectAddDtoList = Lists.newArrayList();
        for (String projectId : projectIds) {
            AddDepProjectDTO addDepProjectDTO = new AddDepProjectDTO();
            addDepProjectDTO.setDepId(depId);
            addDepProjectDTO.setProjectId(projectId);
            depProjectAddDtoList.add(addDepProjectDTO);
        }
        depProjectService.saveBatch(depProjectAddDtoList);
    }

    /**
     * 部门数据入库
     *
     * @param addDepartmentDTO 创建部门入参
     * @return 部门实体对象
     * @author lixing
     * @version V1.0 2021/3/24 10:26 上午
     */
    private Department saveDepartment(AddDepartmentDTO addDepartmentDTO) {
        // 校验部门名称重复
        checkDuplicateDeptName(addDepartmentDTO.getName(), addDepartmentDTO.getParentId());
        // 同一集团下只能有一个类型为集团的部门
        checkDuplicateGroup(addDepartmentDTO);

        Department department = ConvertDepartmentTool.INSTANCE.convertAddDto2Entity(addDepartmentDTO);

        // 设置默认值
        setDefaultValue(department);

        // 获取部门的上级部门，拼装部门的全路径
        String parentId = department.getParentId();
        if (DefaultValueConstants.NO_PARENT_ID.equals(parentId)) {
            department.setFullPath(department.getName());
        } else {
            Department parentDep = queryDepartmentDetail(parentId);
            department.setFullPath(parentDep.getFullPath() + "-" + department.getName());
        }

        save(department);
        return department;
    }


    /**
     * 校验同一集团下是否只有一个类型为集团的部门
     *
     * @param addDepartmentDTO 创建部门入参
     * @author lixing
     * @version V1.0 2021/3/18 7:06 下午
     */
    private void checkDuplicateGroup(AddDepartmentDTO addDepartmentDTO) {
        if (DepTypeEnum.GROUP.getType().equals(addDepartmentDTO.getType())) {
            QueryDepartmentDTO queryDepartmentDTO = new QueryDepartmentDTO();
            queryDepartmentDTO.setType(DepTypeEnum.GROUP.getType());
            List<Department> departments = queryDepartmentList(queryDepartmentDTO);
            if (!CollectionUtils.isEmpty(departments)) {
                throw new BusinessException(ResponseCode.B0300.getCode(),
                        "创建部门失败，同一集团下只能存在一个类型为集团的部门");
            }
        }
    }

    @Override
    public void setDefaultValue(Department department) {
        if (StringUtils.isBlank(department.getResourceFrom())) {
            department.setResourceFrom(ResourceFromEnum.USER_CREATE.getType());
        }

        if (department.getParentId() == null) {
            department.setParentId(DefaultValueConstants.NO_PARENT_ID);
        }

        department.setCreator(DefaultAppContext.getContext().getAccountId());
    }

    /**
     * 校验部门名称是否重复
     *
     * @author lixing
     * @version V1.0 2021/3/12 10:49 上午
     */
    private void checkDuplicateDeptName(String depName, String parentId) {
        List<Department> departments = queryByNameAndParentId(depName, parentId);

        // 处理查询结果
        if (!CollectionUtils.isEmpty(departments)) {
            throw new BusinessException(ResponseCode.B0300.getCode(), "部门名称重复");
        }
    }

    @Override
    public Department queryDepartmentDetail(String id) {
        Department department = getById(id);
        if (department == null) {
            throw new IllegalArgumentException("查看Department详情时发生异常，找不到要查看的记录，id=" + id);
        }
        if (ValidEnum.FALSE.getType().equals(department.getValid())) {
            throw new IllegalArgumentException("部门已被删除");
        }
        return department;
    }

    @Override
    @Transactional
    public void updateDepartment(UpdateDepartmentDTO updateDepartmentDTO) {
        Department department = getById(updateDepartmentDTO.getId());
        if (department == null) {
            throw new IllegalArgumentException("无法找到要更新的部门, id:" + updateDepartmentDTO.getId());
        }
        if (ValidEnum.FALSE.getType().equals(department.getValid())) {
            throw new IllegalArgumentException("部门已被删除");
        }
        if (DepTypeEnum.GROUP.getType().equals(department.getType()) &&
                updateDepartmentDTO.getType() != null &&
                !DepTypeEnum.GROUP.getType().equals(updateDepartmentDTO.getType())) {
            throw new IllegalArgumentException("部门类型不可编辑");
        }
        // 部门名称或者父部门id发生变化，对上级部门进行校验，需要更新子部门的全路径。
        String fullPathAfterChange = getFullPathAfterChange(updateDepartmentDTO, department);
        updateSonDepsFullPath(department.getFullPath(), fullPathAfterChange);

        // 更新部门
        department = ConvertDepartmentTool.INSTANCE.convertUpdateDto2Entity(department, updateDepartmentDTO);
        department.setFullPath(fullPathAfterChange);
        department.setModifier(DefaultAppContext.getContext().getAccountId());
        updateById(department);

        // 更新部门关联的项目
        String depId = department.getId();
        List<String> projectIds = updateDepartmentDTO.getProjectIds();
        updateDepProject(projectIds, depId);

        // 将负责人的工作信息标记为主管
        List<String> managers = updateDepartmentDTO.getManagers();
        // 修改原来部门的负责人工作信息
        QueryWorkResumeDTO queryWorkResumeDTO = new QueryWorkResumeDTO();
        queryWorkResumeDTO.setDepId(department.getId());
        queryWorkResumeDTO.setIsLeader(BoolEnum.TRUE.getType());
        List<WorkResume> workResumes = workResumeService.queryWorkResumeList(queryWorkResumeDTO);
        workResumes.forEach(workResume -> workResume.setIsLeader(BoolEnum.FALSE.getType()));
        workResumeService.batchUpdateWorkResume(workResumes);

        if (!CollectionUtils.isEmpty(managers)) {
            // 修改当前部门负责人的工作信息
            queryWorkResumeDTO.setPersonIds(managers);
            queryWorkResumeDTO.setIsLeader(null);
            workResumes = workResumeService.queryWorkResumeList(queryWorkResumeDTO);
            workResumes.forEach(workResume -> workResume.setIsLeader(BoolEnum.TRUE.getType()));
            workResumeService.batchUpdateWorkResume(workResumes);
        }
    }

    /**
     * 对上级部门进行校验
     * 获取更新后的部门全路径
     *
     * @param updateDepartmentDTO 更新人员入参
     * @param department          更新前的部门对象
     * @return 更新后的部门全路径
     * @author lixing
     * @version V1.0 2021/4/11 8:35 下午
     */
    private String getFullPathAfterChange(UpdateDepartmentDTO updateDepartmentDTO, Department department) {
        String nameBeforeChange = department.getName();
        String fullPathBeforeChange = department.getFullPath();
        String parentIdBeforeChange = department.getParentId();
        String nameAfterChange = nameBeforeChange;
        String fullPathAfterChange = fullPathBeforeChange;
        // 如果部门名称发生变化
        if (StringUtils.isNotBlank(updateDepartmentDTO.getName()) &&
                !nameBeforeChange.equals(updateDepartmentDTO.getName())) {
            nameAfterChange = updateDepartmentDTO.getName();
            String parentId = department.getParentId();
            if (StringUtils.isNotBlank(updateDepartmentDTO.getParentId())) {
                parentId = updateDepartmentDTO.getParentId();
            }
            checkDuplicateDeptName(updateDepartmentDTO.getName(), parentId);
            fullPathAfterChange = getDepFullPathWhenNameChange(
                    nameBeforeChange, fullPathBeforeChange, nameAfterChange);
        }

        // 如果上级部门发生了变化，级联更新自己和子部门的全路径
        String parentIdAfterChange = updateDepartmentDTO.getParentId();
        if (StringUtils.isNotBlank(parentIdAfterChange)) {
            if (!DepConstants.NO_PARENT_ID.equals(parentIdAfterChange)) {
                // 验证选择的上级部门是不是自己的子部门
                Department parentDepAfterChange = queryDepartmentDetail(parentIdAfterChange);
                String parentFullPathAfterChange = parentDepAfterChange.getFullPath();
                if (parentFullPathAfterChange.equals(department.getFullPath()) ||
                        parentFullPathAfterChange.startsWith(department.getFullPath() + DepConstants.FULL_PATH_SEPARATOR)) {
                    throw new IllegalArgumentException("上级部门不能选择自己或自己的子部门（包括子部门的子部门）");
                }
            }

            if (!parentIdAfterChange.equals(parentIdBeforeChange)) {
                fullPathAfterChange = getDepFullPathWhenParentIdChange(
                        nameAfterChange, parentIdAfterChange);
            }
        } else if ("".equals(parentIdAfterChange)) {
            // 如果是将上级部门置空
            fullPathAfterChange = nameAfterChange;
        }

        // 如果部门的全路径发生变化，判断是否存在重复的部门全路径
        if (!fullPathAfterChange.equals(fullPathBeforeChange)) {
            Department tmp = queryDepDetailByFullPath(fullPathAfterChange);
            if (tmp != null) {
                throw new IllegalArgumentException("部门名称已被使用");
            }
        }

        return fullPathAfterChange;
    }

    /**
     * 当部门父部门发生变化时获取更新后的部门的全路径
     *
     * @param depName             更新后的部门名称
     * @param parentIdAfterChange 更新后的父部门id
     * @return 更新后的部门全路径
     * @author lixing
     * @version V1.0 2021/4/11 7:30 下午
     */
    private String getDepFullPathWhenParentIdChange(String depName, String parentIdAfterChange) {
        // 获取更新后的父部门的全路径
        String parentFullPath = "";
        if (!DepConstants.NO_PARENT_ID.equals(parentIdAfterChange)) {
            Department parentDep = queryDepartmentDetail(parentIdAfterChange);
            parentFullPath = parentDep.getFullPath();
        }

        if (StringUtils.isBlank(parentFullPath)) {
            return depName;
        } else {
            return parentFullPath + DepConstants.FULL_PATH_SEPARATOR + depName;
        }
    }

    @Override
    public void updateDepartment(Department department) {
        updateById(department);
    }


    /**
     * 更新部门关联的项目
     *
     * @param projectIds 更新后部门关联的项目
     * @param depId      部门id
     * @author lixing
     * @version V1.0 2021/3/24 10:51 上午
     */
    private void updateDepProject(List<String> projectIds, String depId) {
        // 删除之前部门关联的项目
        depProjectService.deleteByDepId(depId);
        // 插入新的部门关联的项目
        saveDepProject(projectIds, depId);
    }

    /**
     * 当部门名称发生变化时获取更新后的部门全路径
     *
     * @param nameBeforeChange     更新前的部门名称
     * @param fullPathBeforeChange 更新前的部门全路径
     * @param nameAfterChange      更新后的部门名称
     * @return 更新后的部门全路径
     * @author lixing
     * @version V1.0 2021/4/11 7:30 下午
     */
    private String getDepFullPathWhenNameChange(
            String nameBeforeChange, String fullPathBeforeChange, String nameAfterChange) {
        String fullPathAfterChange = fullPathBeforeChange;
        int i = fullPathBeforeChange.lastIndexOf(nameBeforeChange);
        if (i != -1) {
            fullPathAfterChange = fullPathBeforeChange.substring(0, i) + nameAfterChange;
        }
        return fullPathAfterChange;
    }

    /**
     * 更新子部门全路径
     *
     * @param fullPathBeforeChange 更新前部门的全路径
     * @param fullPathAfterChange  更新后部门的全路径
     * @author lixing
     * @version V1.0 2021/4/11 7:27 下午
     */
    private void updateSonDepsFullPath(String fullPathBeforeChange, String fullPathAfterChange) {
        // 根据部门全路径查询子部门
        List<Department> sonDeps = querySonDepsByFullPath(fullPathBeforeChange);
        // 修改子部门全路径
        for (Department sonDep : sonDeps) {
            String tmpFullPath = sonDep.getFullPath();
            tmpFullPath = tmpFullPath.replaceFirst(fullPathBeforeChange + DepConstants.FULL_PATH_SEPARATOR,
                    fullPathAfterChange + DepConstants.FULL_PATH_SEPARATOR);
            sonDep.setFullPath(tmpFullPath);
        }

        updateBatchById(sonDeps);
    }

    @Override
    public List<Department> querySonDepsByFullPath(String fullPath) {
        QueryDepartmentDTO queryDepartmentDTO = new QueryDepartmentDTO();
        queryDepartmentDTO.setFullPath(fullPath + DepConstants.FULL_PATH_SEPARATOR);
        return queryDepartmentList(queryDepartmentDTO);
    }

    @Override
    public List<Department> queryDepsContainsFullPath(String fullPath) {
        QueryDepartmentDTO queryDepartmentDTO = new QueryDepartmentDTO();
        queryDepartmentDTO.setFullPath(fullPath);
        return queryDepartmentList(queryDepartmentDTO);
    }

    @Override
    public Department queryDepDetailByFullPath(String fullPath) {
        QueryWrapper<Department> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(Department.PROP_VALID, ValidEnum.TRUE.getType());
        queryWrapper.eq(Department.PROP_FULL_PATH, fullPath);

        List<Department> deps = list(queryWrapper);
        if (CollectionUtils.isEmpty(deps)) {
            return null;
        } else {
            return deps.get(0);
        }
    }

    @Override
    public Department checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除Department时发生异常，主键为空！");
        }

        Department department = getById(id);

        if (department == null) {
            throw new IllegalArgumentException("删除Department时发生异常，找不到要删除的数据，id:" + id);
        }

        if (ValidEnum.FALSE.getType().equals(department.getValid())) {
            throw new IllegalArgumentException("部门已被删除");
        }

        if (DepTypeEnum.GROUP.getType().equals(department.getType())) {
            throw new IllegalArgumentException("集团不能删除！");
        }

        // 查询部门是否有子部门
        QueryDepartmentDTO queryDepartmentDTO = new QueryDepartmentDTO();
        queryDepartmentDTO.setParentIds(Lists.newArrayList(id));
        List<Department> departments = queryDepartmentList(queryDepartmentDTO);

        if (!CollectionUtils.isEmpty(departments)) {
            throw new IllegalArgumentException("当前部门包含子部门，不可删除");
        }

        PageQueryPersonDTO pageQueryPersonDTO = new PageQueryPersonDTO();
        pageQueryPersonDTO.setPage(PageQueryConstants.DEFAULT_PAGE);
        pageQueryPersonDTO.setSize(PageQueryConstants.DEFAULT_SIZE);
        pageQueryPersonDTO.setDepId(id);
        IPage<ResponsePersonListItemVO> responsePersonListItemVOIPage = personQueryService.pageQueryPerson(pageQueryPersonDTO);
        if (responsePersonListItemVOIPage.getTotal() > 0) {
            throw new IllegalArgumentException("当前部门包含员工，不可删除");
        }
        return department;
    }

    @Override
    @Transactional
    public void deleteDepartment(String id) {
        // 校验部门是否可删除
        Department department = checkDeletable(id);
        department.setValid(ValidEnum.FALSE.getType());
        updateById(department);
    }

    /**
     * 根据名称和父部门id精确查询
     *
     * @param name     部门名称
     * @param parentId 父部门id
     * @return 满足条件的部门列表
     * @author lixing
     * @version V1.0 2021/4/12 4:45 下午
     */
    public List<Department> queryByNameAndParentId(String name, String parentId) {
        QueryWrapper<Department> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(Department.PROP_VALID, ValidEnum.TRUE.getType());
        queryWrapper.eq(Department.PROP_NAME, name);
        queryWrapper.eq(Department.PROP_PARENT_ID, parentId);
        return list(queryWrapper);
    }

    @Override
    public List<Department> queryDepartmentList(QueryDepartmentDTO queryDepartmentDTO) {
        QueryWrapper<Department> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(Department.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, true, Department.PROP_CREATIONTIME);
        if (queryDepartmentDTO != null) {
            if (queryDepartmentDTO.getIds() != null) {
                queryWrapper.in(Department.PROP_ID, queryDepartmentDTO.getIds());
            }

            if (StringUtils.isNotEmpty(queryDepartmentDTO.getName())) {
                queryWrapper.like(Department.PROP_NAME, queryDepartmentDTO.getName());
            }

            if (StringUtils.isNotEmpty(queryDepartmentDTO.getType())) {
                queryWrapper.eq(Department.PROP_TYPE, queryDepartmentDTO.getType());
            }

            if (queryDepartmentDTO.getParentIds() != null) {
                queryWrapper.in(Department.PROP_PARENT_ID, queryDepartmentDTO.getParentIds());
            }

            if (StringUtils.isNotEmpty(queryDepartmentDTO.getResourceFrom())) {
                queryWrapper.eq(Department.PROP_RESOURCE_FROM, queryDepartmentDTO.getResourceFrom());
            }

            if (StringUtils.isNotEmpty(queryDepartmentDTO.getFullPath())) {
                queryWrapper.likeRight(Department.PROP_FULL_PATH, queryDepartmentDTO.getFullPath());
            }
        }
        return list(queryWrapper);
    }

    @Override
    public IPage<Department> pageQueryDepartment(PageQueryDepartmentDTO pageQueryDepartmentDTO) {
        QueryWrapper<Department> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(Department.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<Department> pageParam = new Page<>(pageQueryDepartmentDTO.getPage(), pageQueryDepartmentDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryDepartmentDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, true, Department.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryDepartmentDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        if (StringUtils.isNotEmpty(pageQueryDepartmentDTO.getName())) {
            queryWrapper.like(Department.PROP_NAME, pageQueryDepartmentDTO.getName());
        }

        if (StringUtils.isNotEmpty(pageQueryDepartmentDTO.getType())) {
            queryWrapper.eq(Department.PROP_TYPE, pageQueryDepartmentDTO.getType());
        }

        if (pageQueryDepartmentDTO.getParentIds() != null) {
            queryWrapper.in(Department.PROP_PARENT_ID, pageQueryDepartmentDTO.getParentIds());
        }

        if (StringUtils.isNotEmpty(pageQueryDepartmentDTO.getResourceFrom())) {
            queryWrapper.eq(Department.PROP_RESOURCE_FROM, pageQueryDepartmentDTO.getResourceFrom());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }

    @Override
    public List<Department> getSonDeps(List<Department> parentDeptList) {
        if (CollectionUtils.isEmpty(parentDeptList)) {
            return Lists.newArrayList();
        } else {
            List<String> parentIds = parentDeptList.stream().map(Department::getId).collect(Collectors.toList());
            List<Department> sonDeps = queryDepListByParentIds(parentIds);
            return sonDeps == null ? Lists.newArrayList() : sonDeps;
        }
    }

    @Override
    public List<Department> queryDepListByParentIds(List<String> parentIds) {
        // 拼装查询条件
        QueryWrapper<Department> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(Department.PROP_VALID, ValidEnum.TRUE.getType());
        // 按创建时间正序排序
        queryWrapper.orderBy(true, true, Department.PROP_CREATIONTIME);

        if (CollectionUtils.isEmpty(parentIds)) {
            throw new IllegalArgumentException("根据父节点查询子部门，parentIds为空");
        }
        queryWrapper.in(Department.PROP_PARENT_ID, parentIds);

        return list(queryWrapper);
    }

    @Override
    public List<DepSimpleObjVO> getDepInfoList(List<String> depIds) {
        QueryWrapper<Department> departmentQueryWrapper = new QueryWrapper<>();
        departmentQueryWrapper.eq(Department.PROP_VALID, ValidEnum.TRUE.getType());
        departmentQueryWrapper.in(Department.PROP_ID, depIds);
        List<Department> depList = list(departmentQueryWrapper);
        if (CollectionUtils.isEmpty(depList)) {
            return Lists.newArrayList();
        } else {
            return depList.stream().map(
                    dep -> new DepSimpleObjVO(dep.getId(), dep.getName(), dep.getFullPath())
            ).collect(Collectors.toList());
        }
    }

    @Override
    public Department getGroupDep() {
        QueryDepartmentDTO queryDepartmentDTO = new QueryDepartmentDTO();
        queryDepartmentDTO.setParentIds(Lists.newArrayList(DepConstants.NO_PARENT_ID));
        queryDepartmentDTO.setType(DepTypeEnum.GROUP.getType());
        List<Department> departments = queryDepartmentList(queryDepartmentDTO);
        if (!CollectionUtils.isEmpty(departments)) {
            return departments.get(0);
        } else {
            throw new BusinessException(ResponseCode.B0300.getCode(), "无法获取当前登录人所在的集团部门");
        }
    }

    @Override
    public FullPathInfoVO getFullPathInfo(String depId) {
        FullPathInfoVO fullPathInfoVO = new FullPathInfoVO();
        List<DepSimpleObjVO> resultList = Lists.newArrayList();
        getDepInfoListInFullPath(depId, resultList);
        // 因为部门信息放入的顺序是从后往前，这里做一次反转
        Collections.reverse(resultList);
        fullPathInfoVO.setFullPathInfo(resultList);
        return fullPathInfoVO;
    }

    /**
     * 获取部门全路径中的部门信息列表
     *
     * @param depId      部门id
     * @param resultList 最终要返回的结果
     * @author lixing
     * @version V1.0 2021/4/26 5:12 下午
     */
    private void getDepInfoListInFullPath(String depId, List<DepSimpleObjVO> resultList) {
        if (StringUtils.isNotBlank(depId) && !DepConstants.NO_PARENT_ID.equals(depId)) {
            Department department = getById(depId);
            DepSimpleObjVO depSimpleObjVO = new DepSimpleObjVO();
            depSimpleObjVO.setId(depId);
            depSimpleObjVO.setName(department.getName());
            resultList.add(depSimpleObjVO);

            String parentId = department.getParentId();
            getDepInfoListInFullPath(parentId, resultList);
        }
    }
}
