package com.persagy.fm.depproject.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.base.CaseFormat;
import com.persagy.fm.common.constant.DefaultValueConstants;
import com.persagy.fm.common.constant.enums.ValidEnum;
import com.persagy.fm.common.context.DefaultAppContext;
import com.persagy.fm.common.model.dto.Sort;
import com.persagy.fm.depproject.dao.DepProjectMapper;
import com.persagy.fm.depproject.model.ConvertDepProjectTool;
import com.persagy.fm.depproject.model.DepProject;
import com.persagy.fm.depproject.model.dto.AddDepProjectDTO;
import com.persagy.fm.depproject.model.dto.PageQueryDepProjectDTO;
import com.persagy.fm.depproject.model.dto.QueryDepProjectDTO;
import com.persagy.fm.depproject.model.dto.UpdateDepProjectDTO;
import com.persagy.fm.depproject.service.IDepProjectService;
import com.persagy.fm.saas.project.model.vo.ProjectTreeItemVO;
import com.persagy.fm.saas.project.service.ISaasProjectService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 部门项目(DepProject) service层
 *
 * @author lixing
 * @version V1.0 2021-03-24 10:08:12
 */
@Service
public class DepProjectServiceImpl extends ServiceImpl<DepProjectMapper, DepProject>
        implements IDepProjectService {
    @Autowired
    ISaasProjectService saasProjectService;

    @Override
    public List<ProjectTreeItemVO> queryDepProjectTree(List<String> parentDepIds) {
        // 父部门id 为0或空, 加载整颗区域项目树
        if (CollectionUtils.isEmpty(parentDepIds) ||
                parentDepIds.contains(DefaultValueConstants.NO_PARENT_ID)) {
            return saasProjectService.queryAllProjectTree();
        }
        // 部门关联的项目
        QueryDepProjectDTO queryDepProjectDTO = new QueryDepProjectDTO();
        queryDepProjectDTO.setDepIds(parentDepIds);
        List<DepProject> depProjectList = queryDepProjectList(queryDepProjectDTO);
        List<String> projectIds = depProjectList.stream().
                map(DepProject::getProjectId).
                distinct().
                collect(Collectors.toList());
        return saasProjectService.queryProjectTree(projectIds);
    }

    @Override
    public void saveBatch(List<AddDepProjectDTO> addDepProjectDTOList) {
        List<DepProject> depProjectList = ConvertDepProjectTool.INSTANCE.convert2EntityList(addDepProjectDTOList);
        for (DepProject depProject : depProjectList) {
            setDefaultValue(depProject);
        }
        saveBatch(depProjectList);
    }

    @Override
    public void deleteByDepId(String depId) {
        QueryDepProjectDTO queryDepProjectDTO = new QueryDepProjectDTO();
        queryDepProjectDTO.setDepId(depId);
        List<DepProject> depProjectList = queryDepProjectList(queryDepProjectDTO);
        List<String> idList = depProjectList.stream().map(DepProject::getId).collect(Collectors.toList());
        removeByIds(idList);
    }

    /**
     * 创建部门项目
     *
     * @return Long 部门项目主键
     * @author lixing
     * @version V1.0 2021-03-24 10:08:12
     */
    @Override
    public String createDepProject(AddDepProjectDTO addDepProjectDTO) {
        DepProject depProject = ConvertDepProjectTool.INSTANCE.convertAddDto2Entity(addDepProjectDTO);
        // 设置默认值
        setDefaultValue(depProject);
        save(depProject);
        return depProject.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param depProject 部门项目实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(DepProject depProject) {
        depProject.setCreator(DefaultAppContext.getContext().getAccountId());
    }

    /**
     * 部门项目详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-03-24 10:08:12
     */
    @Override
    public DepProject queryDepProjectDetail(String id) {
        DepProject depProject = getById(id);
        if (depProject == null) {
            throw new IllegalArgumentException("查看DepProject详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return depProject;
    }

    /**
     * 更新部门项目
     *
     * @author lixing
     * @version V1.0 2021-03-24 10:08:12
     */
    @Override
    public void updateDepProject(UpdateDepProjectDTO updateDepProjectDTO) {
        DepProject depProject = getById(updateDepProjectDTO.getId());
        depProject = ConvertDepProjectTool.INSTANCE.convertUpdateDto2Entity(depProject, updateDepProjectDTO);
        depProject.setModifier(DefaultAppContext.getContext().getAccountId());
        updateById(depProject);
    }

    /**
     * 校验部门项目是否可删除
     *
     * @param id 部门项目主键
     * @return 部门项目do类
     * @author lixing
     * @version V1.0 2021-03-24 10:08:12
     */
    public DepProject checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除DepProject时发生异常，主键为空");
        }

        DepProject depProject = getById(id);

        if (depProject == null) {
            throw new IllegalArgumentException("删除DepProject时发生异常，找不到要删除的数据，id:" + id);
        }

        return depProject;
    }

    /**
     * 删除部门项目
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-03-24 10:08:12
     */
    @Override
    public void deleteDepProject(String id) {
        // 校验是否可删除
        DepProject depProject = checkDeletable(id);

        depProject.setValid(ValidEnum.FALSE.getType());
        updateById(depProject);
    }

    /**
     * 查询部门项目
     *
     * @return List<DepProject>
     * @author lixing
     * @version V1.0 2021-03-24 10:08:12
     */
    @Override
    public List<DepProject> queryDepProjectList(QueryDepProjectDTO queryDepProjectDTO) {
        QueryWrapper<DepProject> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(DepProject.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, DepProject.PROP_CREATIONTIME);

        if (queryDepProjectDTO != null) {

            if (queryDepProjectDTO.getDepId() != null) {
                queryWrapper.eq(DepProject.PROP_DEP_ID, queryDepProjectDTO.getDepId());
            }

            if (queryDepProjectDTO.getDepIds() != null) {
                queryWrapper.in(DepProject.PROP_DEP_ID, queryDepProjectDTO.getDepIds());
            }

            if (queryDepProjectDTO.getProjectId() != null) {
                queryWrapper.eq(DepProject.PROP_PROJECT_ID, queryDepProjectDTO.getProjectId());
            }

        }

        return list(queryWrapper);
    }

    /**
     * 分页查询部门项目
     *
     * @return IPage<DepProject>
     * @author lixing
     * @version V1.0 2021-03-24 10:08:12
     */
    @Override
    public IPage<DepProject> pageQueryDepProject(PageQueryDepProjectDTO pageQueryDepProjectDTO) {
        QueryWrapper<DepProject> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(DepProject.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<DepProject> pageParam = new Page<>(pageQueryDepProjectDTO.getPage(), pageQueryDepProjectDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryDepProjectDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, DepProject.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryDepProjectDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        if (pageQueryDepProjectDTO.getDepId() != null) {
            queryWrapper.eq(DepProject.PROP_DEP_ID, pageQueryDepProjectDTO.getDepId());
        }

        if (pageQueryDepProjectDTO.getProjectId() != null) {
            queryWrapper.eq(DepProject.PROP_PROJECT_ID, pageQueryDepProjectDTO.getProjectId());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }
}
