package com.persagy.fm.profession.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.persagy.fm.common.constant.enums.BoolEnum;
import com.persagy.fm.common.constant.enums.ValidEnum;
import com.persagy.fm.common.context.DefaultAppContext;
import com.persagy.fm.common.model.vo.SimpleObjVO;
import com.persagy.fm.person.model.Person;
import com.persagy.fm.person.service.IPersonQueryService;
import com.persagy.fm.profession.dao.ProfessionMapper;
import com.persagy.fm.profession.model.ConvertProfessionTool;
import com.persagy.fm.profession.model.Profession;
import com.persagy.fm.profession.model.dto.AddProfessionDTO;
import com.persagy.fm.profession.model.dto.UpdateProfessionDTO;
import com.persagy.fm.profession.service.IProfessionService;
import org.apache.commons.lang3.StringUtils;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 专业(Profession) service层
 *
 * @author lixing
 * @version V1.0 2021-03-19 16:38:19
 */
@Service
public class ProfessionServiceImpl extends ServiceImpl<ProfessionMapper, Profession>
        implements IProfessionService {
    @Autowired
    IPersonQueryService personQueryService;

    /**
     * 创建专业
     *
     * @return Long 专业主键
     * @author lixing
     * @version V1.0 2021-03-19 16:38:19
     */
    @Override
    public String createProfession(AddProfessionDTO addProfessionDTO) {
        checkDuplicateName(addProfessionDTO.getName());
        Profession profession = ConvertProfessionTool.INSTANCE.convertAddDto2Entity(addProfessionDTO);
        // 设置默认值
        setDefaultValue(profession);
        save(profession);
        return profession.getId();
    }

    /**
     * 校验专业名称是否重复
     *
     * @param name 名称
     * @author lixing
     * @version V1.0 2021/4/21 2:26 下午
     */
    private void checkDuplicateName(String name) {
        List<Profession> professions = queryProfessionListByName(name);
        if (!CollectionUtils.isEmpty(professions)) {
            throw new IllegalArgumentException("专业名称已被使用");
        }
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param profession 专业实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(Profession profession) {
        profession.setCreator(DefaultAppContext.getContext().getAccountId());
        if (profession.getEditable() == null) {
            profession.setEditable(BoolEnum.TRUE.getType());
        }
    }

    /**
     * 更新专业
     *
     * @author lixing
     * @version V1.0 2021-03-19 16:38:19
     */
    @Override
    public void updateProfession(UpdateProfessionDTO updateProfessionDTO) {
        Profession profession = getById(updateProfessionDTO.getId());
        // 校验名称重复
        if (StringUtils.isNotBlank(updateProfessionDTO.getName()) &&
                !updateProfessionDTO.getName().equals(profession.getName())) {
            checkDuplicateName(updateProfessionDTO.getName());
        }
        profession = ConvertProfessionTool.INSTANCE.convertUpdateDto2Entity(profession, updateProfessionDTO);
        profession.setModifier(DefaultAppContext.getContext().getAccountId());
        updateById(profession);
    }

    @Override
    public Profession checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除Profession时发生异常，主键为空");
        }

        Profession profession = getById(id);

        if (profession == null) {
            throw new IllegalArgumentException("删除Profession时发生异常，找不到要删除的数据，id:" + id);
        }

        // 如果专业被人员使用，专业不可删除
        List<Person> people = personQueryService.queryByProfessionId(id);
        if (!CollectionUtils.isEmpty(people)) {
            throw new IllegalArgumentException("被引用的专业不可删除");
        }

        return profession;
    }

    /**
     * 删除专业
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-03-19 16:38:19
     */
    @Override
    public void deleteProfession(String id) {
        // 校验是否可删除
        Profession profession = checkDeletable(id);

        profession.setValid(ValidEnum.FALSE.getType());
        updateById(profession);
    }

    /**
     * 查询专业
     *
     * @return List<Profession>
     * @author lixing
     * @version V1.0 2021-03-19 16:38:19
     */
    @Override
    public List<Profession> queryProfessionList() {
        QueryWrapper<Profession> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(Profession.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, Profession.PROP_CREATIONTIME);
        return list(queryWrapper);
    }

    @Override
    public List<SimpleObjVO> queryProfessionInfoByProfessionIds(List<String> professionIdList) {
        if (CollectionUtils.isEmpty(professionIdList)) {
            return Lists.newArrayList();
        }
        QueryWrapper<Profession> queryWrapper = new QueryWrapper<>();
        queryWrapper.in(Profession.PROP_ID, professionIdList);
        List<Profession> professions = list(queryWrapper);
        return professions.stream().map(profession-> {
            SimpleObjVO simpleObjVO = new SimpleObjVO();
            simpleObjVO.setId(profession.getId());
            simpleObjVO.setName(profession.getName());
            return simpleObjVO;
        }).collect(Collectors.toList());
    }

    /**
     * 根据专业名称查询专业
     *
     * @return List<Profession>
     * @author lixing
     * @version V1.0 2021-03-19 16:38:19
     */
    private List<Profession> queryProfessionListByName(String name) {
        if (StringUtils.isBlank(name)) {
            return null;
        }
        QueryWrapper<Profession> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(Profession.PROP_VALID, ValidEnum.TRUE.getType());
        queryWrapper.eq(Profession.PROP_NAME, name);
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, Profession.PROP_CREATIONTIME);
        return list(queryWrapper);
    }
}
