package com.persagy.fm.saas.project.service.impl;

import com.persagy.fm.common.constant.enums.BoolEnum;
import com.persagy.fm.common.response.FmResponseContent;
import com.persagy.fm.common.response.FmResponseItem;
import com.persagy.fm.saas.project.client.SaasProjectClient;
import com.persagy.fm.saas.project.model.SaasArea;
import com.persagy.fm.saas.project.model.SaasProject;
import com.persagy.fm.saas.project.model.dto.QueryAllowSaasAreaProjectTreeDTO;
import com.persagy.fm.saas.project.model.dto.QuerySaasAreaDTO;
import com.persagy.fm.saas.project.model.dto.QuerySaasAreaProjectTreeDTO;
import com.persagy.fm.saas.project.model.dto.QuerySaasProjectDTO;
import com.persagy.fm.saas.project.model.vo.ProjectTreeItemVO;
import com.persagy.fm.saas.project.model.vo.SaasAreaProjectTreeItemVO;
import com.persagy.fm.saas.project.model.vo.SaasAreaProjectTreeVO;
import com.persagy.fm.saas.project.model.vo.SaasProjectTreeItemVO;
import com.persagy.fm.saas.project.service.ISaasProjectService;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.List;

/**
 * 项目service层
 *
 * @author lixing
 * @version V1.0 2021/3/22 2:05 下午
 **/
@Service
public class SaasProjectServiceImpl implements ISaasProjectService {
    @Autowired
    SaasProjectClient saasProjectClient;

    @Override
    public List<ProjectTreeItemVO> queryProjectTree(List<String> projectIds) {
        if (CollectionUtils.isEmpty(projectIds)) {
            return Lists.newArrayList();
        }
        // 查询运维系统区域项目树
        QueryAllowSaasAreaProjectTreeDTO queryAllowSaasAreaProjectTreeDTO = new QueryAllowSaasAreaProjectTreeDTO();
        queryAllowSaasAreaProjectTreeDTO.setDefaultValue();
        queryAllowSaasAreaProjectTreeDTO.setProjectIds(projectIds);
        FmResponseContent<List<SaasAreaProjectTreeVO>> responseItem = saasProjectClient.queryAllowAreaProjectTree(queryAllowSaasAreaProjectTreeDTO);
        // 查询结果转换
        return convert2ProjectTreeItemList(responseItem);
    }

    @Override
    public List<ProjectTreeItemVO> queryAllProjectTree() {
        // 查询运维系统区域项目树
        QuerySaasAreaProjectTreeDTO querySaasAreaProjectTreeDTO = new QuerySaasAreaProjectTreeDTO();
        querySaasAreaProjectTreeDTO.setDefaultValue();
        FmResponseContent<List<SaasAreaProjectTreeVO>> responseContent = saasProjectClient.queryAreaProjectTree(querySaasAreaProjectTreeDTO);
        // 查询结果转换
        return convert2ProjectTreeItemList(responseContent);
    }

    /**
     * 将运维系统区域项目树，转换为fm区域项目树
     *
     * @param responseItem 运维系统区域项目树查询结果
     * @return fm区域项目树
     * @author lixing
     * @version V1.0 2021/3/29 6:30 下午
     */
    private List<ProjectTreeItemVO> convert2ProjectTreeItemList(FmResponseContent<List<SaasAreaProjectTreeVO>> responseItem) {
        List<SaasAreaProjectTreeVO> content = responseItem.getContent();
        if (CollectionUtils.isEmpty(content)) {
            return Lists.newArrayList();
        }
        SaasAreaProjectTreeVO saasAreaProjectTreeVO = content.get(0);
        if (saasAreaProjectTreeVO == null) {
            return Lists.newArrayList();
        }
        List<SaasAreaProjectTreeItemVO> areas = saasAreaProjectTreeVO.getAreas();
        if (CollectionUtils.isEmpty(areas)) {
            return Lists.newArrayList();
        }
        List<ProjectTreeItemVO> result = Lists.newArrayList();
        for (SaasAreaProjectTreeItemVO area : areas) {
            ProjectTreeItemVO projectTreeItemVO = convert2ProjectTreeItemVO(area);
            result.add(projectTreeItemVO);
        }
        return result;
    }

    /**
     * 将项目对象转换为项目树节点对象
     *
     * @param project 项目对象
     * @return 项目树节点对象
     * @author lixing
     * @version V1.0 2021/3/26 6:21 下午
     */
    private ProjectTreeItemVO convert2ProjectTreeItemVO(SaasProjectTreeItemVO project) {
        ProjectTreeItemVO projectTreeItemVO = new ProjectTreeItemVO();
        projectTreeItemVO.setName(project.getProjectName());
        projectTreeItemVO.setId(project.getProjectId());
        projectTreeItemVO.setAreaId(project.getAreaId());
        projectTreeItemVO.setIsProject(BoolEnum.TRUE.getType());
        return projectTreeItemVO;
    }

    /**
     * 将区域对象转换为项目树节点对象
     *
     * @param area 区域对象
     * @return 项目树节点对象
     * @author lixing
     * @version V1.0 2021/3/26 6:18 下午
     */
    private ProjectTreeItemVO convert2ProjectTreeItemVO(SaasAreaProjectTreeItemVO area) {
        ProjectTreeItemVO projectTreeItemVO = new ProjectTreeItemVO();
        projectTreeItemVO.setName(area.getAreaName());
        projectTreeItemVO.setId(area.getId());
        projectTreeItemVO.setAreaId(area.getId());
        projectTreeItemVO.setIsProject(BoolEnum.FALSE.getType());

        // 如果区域下包含项目
        List<SaasProjectTreeItemVO> projects = area.getProjects();
        if (!CollectionUtils.isEmpty(projects)) {
            List<ProjectTreeItemVO> projectItems = Lists.newArrayList();
            for (SaasProjectTreeItemVO project : projects) {
                ProjectTreeItemVO projectItem = convert2ProjectTreeItemVO(project);
                projectItems.add(projectItem);
            }
            addChildren(projectTreeItemVO, projectItems);
        }

        // 如果区域下包含区域子集
        List<SaasAreaProjectTreeItemVO> areaChildren = area.getChildrens();
        if (!CollectionUtils.isEmpty(areaChildren)) {
            List<ProjectTreeItemVO> areaItems = Lists.newArrayList();
            for (SaasAreaProjectTreeItemVO areaChild : areaChildren) {
                ProjectTreeItemVO areaItem = convert2ProjectTreeItemVO(areaChild);
                areaItems.add(areaItem);
            }
            addChildren(projectTreeItemVO, areaItems);
        }
        return projectTreeItemVO;
    }

    /**
     * 给项目树父节点添加子节点
     *
     * @param parentNode 父节点
     * @param children2Add 子节点列表
     * @author lixing
     * @version V1.0 2021/3/27 2:05 下午
     */
    private void addChildren(ProjectTreeItemVO parentNode, List<ProjectTreeItemVO> children2Add) {
        List<ProjectTreeItemVO> children = parentNode.getChildren();
        if (CollectionUtils.isEmpty(children)) {
            children = Lists.newArrayList();
        }
        children.addAll(children2Add);
        parentNode.setChildren(children);
    }

    @Override
    public List<SaasProject> queryProjectList(List<String> projectIds) {
        QuerySaasProjectDTO querySaasProjectDTO = new QuerySaasProjectDTO();
        querySaasProjectDTO.setDefaultValue();
        querySaasProjectDTO.setProjectIds(projectIds);
        FmResponseContent<List<SaasProject>> listFmResponseContent = saasProjectClient.queryProjectList(querySaasProjectDTO);
        return listFmResponseContent.getContent();
    }
}
